import os
import time
import threading
import tkinter as tk
from tkinter import ttk
from PIL import Image, ImageTk
import pyvisa
import numpy as np

# Konfiguration
#OSZI_IP = "192.168.1.54"
# Prompt for IP address via CLI
OSZI_IP = input("Enter RIGOL MSO5000 IP address: ").strip()
if not OSZI_IP:
    print("❌ No IP address provided. Exiting.")
    exit(1)

BILDPFAD = "/tmp/oszi_screenshot.png"
INTERVALL_BILD = 2      # Sekunden für Bild-Update
INTERVALL_SCPI = 4      # Sekunden für SCPI-Update (langsamer für hacked FW)
WAV_POINTS = 1200       # Anzahl Samples für VPP-Berechnung

# Commands that are known to timeout on hacked firmware
BLACKLISTED_COMMANDS = [
    ":SYSTem:UPTime?",
    ":SYSTem:TEMPerature?", 
    ":SYSTem:OPTions?",
    ":TIMebase:HREFerence?",
    ":ACQuire:MODE?",
    ":ACQuire:TYPE?",
    ":TRIGger:HOLDoff?",
    ":WAVeform:YINCrement?",
    ":WAVeform:YORigin?",
    ":WAVeform:YREFerence?"
]

# Dark theme colors
DARK_BG = "#1a1a1a"      # Dark background
DARK_FG = "#ffffff"      # White text
DARK_SELECT = "#333333"   # Selection color
DARK_TAB_BG = "#2d2d2d"   # Tab background

# GUI setup
root = tk.Tk()
root.title("RIGOL MSO5000 Live Monitor (Hacked FW Compatible)")
root.geometry("1200x800")
root.minsize(800, 600)  # Minimum size for usability
root.configure(bg=DARK_BG)

# Configure style for dark theme
style = ttk.Style()
style.theme_use('clam')
style.configure('TNotebook', background=DARK_BG, borderwidth=0)
style.configure('TNotebook.Tab', background=DARK_TAB_BG, foreground=DARK_FG, 
                padding=[10, 5], borderwidth=1)
style.map('TNotebook.Tab', background=[('selected', DARK_SELECT)])
style.configure('TFrame', background=DARK_BG)

# Screenshot display with dynamic sizing
global img_label
img_label = tk.Label(root, bg=DARK_BG)
img_label.pack(pady=5)

# Create notebook for tabbed info display
notebook = ttk.Notebook(root)
notebook.pack(fill="both", expand=True, padx=10, pady=5)

# Tab 1: System Info
system_frame = ttk.Frame(notebook)
notebook.add(system_frame, text="System Info")
system_text = tk.StringVar()
system_label = tk.Label(
    system_frame,
    textvariable=system_text,
    font=("Courier", 10),
    justify="left",
    anchor="nw",
    wraplength=1180,
    bg=DARK_BG,
    fg=DARK_FG
)
system_label.pack(fill="both", expand=True, padx=5, pady=5)

# Add scrollbar for system info
system_canvas = tk.Canvas(system_frame, bg=DARK_BG, highlightthickness=0)
system_scrollbar = ttk.Scrollbar(system_frame, orient="vertical", command=system_canvas.yview)
system_scrollable_frame = tk.Frame(system_canvas, bg=DARK_BG)

system_scrollable_frame.bind(
    "<Configure>",
    lambda e: system_canvas.configure(scrollregion=system_canvas.bbox("all"))
)

system_canvas.create_window((0, 0), window=system_scrollable_frame, anchor="nw")
system_canvas.configure(yscrollcommand=system_scrollbar.set)

# Replace the simple label with scrollable version
system_label.destroy()
system_label = tk.Label(
    system_scrollable_frame,
    textvariable=system_text,
    font=("Courier", 9),  # Slightly smaller font
    justify="left",
    anchor="nw",
    bg=DARK_BG,
    fg=DARK_FG
)
system_label.pack(fill="both", expand=True, padx=5, pady=5)

system_canvas.pack(side="left", fill="both", expand=True, padx=5, pady=5)
system_scrollbar.pack(side="right", fill="y")

# Tab 2: Channel Data
channel_frame = ttk.Frame(notebook)
notebook.add(channel_frame, text="Channel Data")
channel_text = tk.StringVar()
channel_label = tk.Label(
    channel_frame,
    textvariable=channel_text,
    font=("Courier", 9),  # Slightly smaller font
    justify="left",
    anchor="nw",
    bg=DARK_BG,
    fg=DARK_FG
)
channel_label.pack(fill="both", expand=True, padx=5, pady=5)

# Add scrollbar for channel info
channel_canvas = tk.Canvas(channel_frame, bg=DARK_BG, highlightthickness=0)
channel_scrollbar = ttk.Scrollbar(channel_frame, orient="vertical", command=channel_canvas.yview)
channel_scrollable_frame = tk.Frame(channel_canvas, bg=DARK_BG)

channel_scrollable_frame.bind(
    "<Configure>",
    lambda e: channel_canvas.configure(scrollregion=channel_canvas.bbox("all"))
)

channel_canvas.create_window((0, 0), window=channel_scrollable_frame, anchor="nw")
channel_canvas.configure(yscrollcommand=channel_scrollbar.set)

# Replace the simple label with scrollable version
channel_label.destroy()
channel_label = tk.Label(
    channel_scrollable_frame,
    textvariable=channel_text,
    font=("Courier", 9),  # Slightly smaller font
    justify="left",
    anchor="nw",
    bg=DARK_BG,
    fg=DARK_FG
)
channel_label.pack(fill="both", expand=True, padx=5, pady=5)

channel_canvas.pack(side="left", fill="both", expand=True, padx=5, pady=5)
channel_scrollbar.pack(side="right", fill="y")

# Tab 3: Debug Log
debug_frame = ttk.Frame(notebook)
notebook.add(debug_frame, text="Debug Log")
debug_text = tk.Text(debug_frame, font=("Courier", 9), height=10, 
                     bg=DARK_BG, fg=DARK_FG, insertbackground=DARK_FG,
                     selectbackground=DARK_SELECT)
debug_scrollbar = ttk.Scrollbar(debug_frame, orient="vertical", command=debug_text.yview)
debug_text.configure(yscrollcommand=debug_scrollbar.set)
debug_text.pack(side="left", fill="both", expand=True, padx=5, pady=5)
debug_scrollbar.pack(side="right", fill="y")

# Configure scrollbar colors
style.configure('Vertical.TScrollbar', background=DARK_TAB_BG, troughcolor=DARK_BG)

# Global variables for data sharing
scpi_data = {
    'connected': False,
    'last_update': 'Never',
    'errors': 0,
    'system_info': {},
    'channel_data': {},
    'debug_log': []
}

def log_debug(message):
    """Add message to debug log"""
    timestamp = time.strftime("%H:%M:%S")
    scpi_data['debug_log'].append(f"[{timestamp}] {message}")
    # Keep only last 100 entries
    if len(scpi_data['debug_log']) > 100:
        scpi_data['debug_log'].pop(0)
    
    # Update debug text widget
    debug_text.delete(1.0, tk.END)
    debug_text.insert(tk.END, "\n".join(scpi_data['debug_log'][-50:]))  # Show last 50 entries
    debug_text.see(tk.END)

def safe_scpi_query(scope, command, default="N/A"):
    """Safely execute SCPI query with error handling"""
    if command in BLACKLISTED_COMMANDS:
        log_debug(f"⚠️ Skipped blacklisted SCPI command: {command}")
        return default
    
    try:
        result = scope.query(command).strip()
        #log_debug(f"✅ SCPI: {command} => {result}")
        return result if result else default
    except Exception as e:
        log_debug(f"❌ SCPI Error [{command}]: {e}")
        if "TMO" in str(e) or "Timeout" in str(e):
            BLACKLISTED_COMMANDS.append(command)
            log_debug(f"⛔ Added to blacklist due to timeout: {command}")
        return default

def safe_scpi_write(scope, command):
    """Safely execute SCPI write with error handling"""
    try:
        scope.write(command)
        return True
    except Exception as e:
        log_debug(f"SCPI Write Error [{command}]: {e}")
        return False

def screenshot_loop():
    """Continuously capture screenshots from VNC"""
    tmpfile = BILDPFAD + ".tmp.png"
    while True:
        try:
            os.system(f"vncdo -s {OSZI_IP} capture {tmpfile}")
            if os.path.exists(tmpfile):
                os.replace(tmpfile, BILDPFAD)  # atomic replace
        except Exception as e:
            log_debug(f"Screenshot Error: {e}")
        time.sleep(INTERVALL_BILD)

def update_image():
    """Update the displayed screenshot with dynamic sizing"""
    try:
        if os.path.exists(BILDPFAD):
            with open(BILDPFAD, "rb") as f:
                try:
                    img = Image.open(f)
                    img.load()  # Ensure full image read
                except Exception as e:
                    raise IOError(f"Image Load Error (possibly truncated): {e}")

            # Resize logic...
            window_width = root.winfo_width()
            window_height = root.winfo_height()
            available_height = max(300, window_height - 250)
            available_width = max(600, window_width - 50)

            img_ratio = img.width / img.height
            if available_width / available_height > img_ratio:
                new_height = available_height
                new_width = int(available_height * img_ratio)
            else:
                new_width = available_width
                new_height = int(available_width / img_ratio)

            new_width = max(400, new_width)
            new_height = max(240, new_height)

            img = img.resize((new_width, new_height), Image.Resampling.LANCZOS)
            img_tk = ImageTk.PhotoImage(img)
            img_label.config(image=img_tk)
            img_label.image = img_tk

    except Exception as e:
        log_debug(str(e))

    root.after(INTERVALL_BILD * 1000, update_image)

def get_channel_waveform_data(scope, channel, use_simple_calc=True):
    """Get waveform data for a specific channel - simplified for hacked firmware"""
    try:
        if not safe_scpi_write(scope, f":WAVeform:SOURce CHAN{channel}"):
            return None, None, None
        
        # Try to get waveform data
        raw = scope.query_binary_values(":WAVeform:DATA?", datatype='B', container=np.array)
        if len(raw) == 0:
            return None, None, None
        
        if use_simple_calc:
            # Simple calculation without scaling factors (for hacked firmware)
            # Assume 8-bit data (0-255) and approximate voltage conversion
            vpp_raw = raw.max() - raw.min()
            vavg_raw = raw.mean()
            vrms_raw = np.sqrt(np.mean((raw - vavg_raw)**2))
            
            # Rough approximation: each bit ≈ few mV depending on V/div setting
            # User will need to interpret these relative values
            return vpp_raw, vavg_raw, vrms_raw
        else:
            # This won't work on hacked firmware due to scaling commands timing out
            return None, None, None
            
    except Exception as e:
        log_debug(f"Waveform data error CH{channel}: {e}")
        return None, None, None

def scpi_loop():
    """Main SCPI communication loop - optimized for hacked firmware"""
    scope = None
    
    while True:
        try:
            # Try to connect if not connected
            if scope is None:
                try:
                    rm = pyvisa.ResourceManager()
                    scope = rm.open_resource(f'TCPIP0::{OSZI_IP}::INSTR')
                    scope.timeout = 3000  # Even shorter timeout for hacked firmware
                    scope.chunk_size = 102400
                    scpi_data['connected'] = True
                    log_debug("SCPI Connection established")
                    
                    # Minimal setup for waveform transfer
                    safe_scpi_write(scope, ":WAVeform:FORMat BYTE")
                    safe_scpi_write(scope, ":WAVeform:MODE NORM")
                    safe_scpi_write(scope, f":WAVeform:POINts {WAV_POINTS}")
                    
                except Exception as e:
                    log_debug(f"SCPI Connection failed: {e}")
                    scpi_data['connected'] = False
                    scpi_data['errors'] += 1
                    time.sleep(5)
                    continue
            
            # Get basic system information (only what works)
            now = time.strftime("%Y-%m-%d %H:%M:%S")
            idn = safe_scpi_query(scope, "*IDN?", "Unknown Device")
            
            # Parse IDN
            parts = idn.split(',')
            model = parts[1] if len(parts) > 1 else "MSO5000"
            fw_version = parts[3] if len(parts) > 3 else "Hacked"
            
            # Basic system info (skip problematic commands)
            system_info = {
                'model': model,
                'firmware': fw_version,
                'uptime': "N/A (Hacked FW)",
                'temperature': "N/A (Hacked FW)",
                'options': "N/A (Hacked FW)"
            }
            
            # Get trigger information (only basic commands)
            trig_stat = safe_scpi_query(scope, ":TRIGger:STATus?", "Unknown")
            trig_mode = safe_scpi_query(scope, ":TRIGger:MODE?", "Unknown")
            trig_source = safe_scpi_query(scope, ":TRIGger:EDGE:SOURce?", "Unknown")
            trig_lvl = safe_scpi_query(scope, ":TRIGger:EDGE:LEVel?", "N/A")
            trig_slope = safe_scpi_query(scope, ":TRIGger:EDGE:SLOPe?", "Unknown")
            log_debug(
                f"Trigger={trig_stat} ({trig_mode} on {trig_source}) | Level={trig_lvl}V | Slope={trig_slope}"
            )
            
            # Get timebase information (skip problematic commands)
            hscale = safe_scpi_query(scope, ":TIMebase:SCALe?", "N/A")
            hpos = safe_scpi_query(scope, ":TIMebase:HREFerence:POSition?", "N/A")
            
            # Get acquisition information (skip problematic commands)
            srate = safe_scpi_query(scope, ":ACQuire:SRATe?", "N/A")
            mdepth = safe_scpi_query(scope, ":ACQuire:MDEPth?", "N/A")

            log_debug(
                f"Timebase={hscale}s/div | Position={hpos}s | SR={srate} Sa/s | MemDepth={mdepth} pts"
            )
            
            # Get channel information
            channel_data = {}
            for i in range(1, 5):
                log_debug(f"🔍 Checking Channel {i}")
                disp = safe_scpi_query(scope, f":CHANnel{i}:DISPlay?", "0")
                if disp in ('1', 'ON'):
                    coupl = safe_scpi_query(scope, f":CHANnel{i}:COUPling?", "DC")
                    bw = safe_scpi_query(scope, f":CHANnel{i}:BWLimit?", "OFF")
                    vscale = safe_scpi_query(scope, f":CHANnel{i}:SCALe?", "1.0")
                    voff = safe_scpi_query(scope, f":CHANnel{i}:OFFSet?", "0.0")
                    probe = safe_scpi_query(scope, f":CHANnel{i}:PROBe?", "1")
                    units = safe_scpi_query(scope, f":CHANnel{i}:UNITs?", "V")
                    impedance = safe_scpi_query(scope, f":CHANnel{i}:IMPedance?", "N/A")
                    inverted = safe_scpi_query(scope, f":CHANnel{i}:INVert?", "OFF")
                    vernier = safe_scpi_query(scope, f":CHANnel{i}:VERNier?", "OFF")
                    
                    # Get waveform statistics (simplified)
                    vpp, vavg, vrms = get_channel_waveform_data(scope, i)
      
                    log_debug(
                        f"CH{i}: DISP=ON, Coupling={coupl}, BW={bw}, Scale={vscale} {units}/div, "
                        f"Offset={voff} {units}, Probe={probe}x, Imp={impedance}, Inv={inverted}, Vernier={vernier}"
                    )
              
                    channel_data[i] = {
                        'displayed': True,
                        'coupling': coupl,
                        'bandwidth': bw,
                        'scale': vscale,
                        'offset': voff,
                        'probe': probe,
                        'vpp_raw': f"{vpp:.0f}" if vpp is not None else "N/A",
                        'vavg_raw': f"{vavg:.0f}" if vavg is not None else "N/A",
                        'vrms_raw': f"{vrms:.0f}" if vrms is not None else "N/A"
                    }
            
            # Update global data
            scpi_data['last_update'] = now
            scpi_data['system_info'] = system_info
            scpi_data['channel_data'] = channel_data
            
            # Format system info display
            system_display = (
                f"Connected: {scpi_data['connected']} | Last Update: {now}\n"
                f"Model: {model} | Firmware: {fw_version}\n"
                f"Note: Some features disabled due to hacked firmware\n\n"
                f"Trigger: {trig_stat} | Mode: {trig_mode} | Source: {trig_source}\n"
                f"Level: {trig_lvl}V | Slope: {trig_slope}\n"
                f"Timebase: {hscale}s/div | Position: {hpos}s\n"
                f"Sample Rate: {srate}Sa/s | Memory Depth: {mdepth}pts\n"
                f"Units: {units} | Impedance: {impedance}Ω | Inverted: {inverted} | Vernier: {vernier}\n\n"
                f"SCPI Errors: {scpi_data['errors']}\n"
                f"Blacklisted Commands: {len(BLACKLISTED_COMMANDS)}\n"
            )
            system_text.set(system_display)
            
            # Format channel info display
            channel_display = "📡 CHANNEL STATUS:\n" + "="*80 + "\n"
            for i in range(1, 5):
                if i in channel_data:
                    ch = channel_data[i]
                    channel_display += (
                        f"CH{i}: ✅ ON  | {ch['coupling']} | BW:{ch['bandwidth']} | "
                        f"Scale:{ch['scale']}V/div | Offset:{ch['offset']}V\n"
                        f"      Probe:{ch['probe']}x\n"
                        f"      📈 Raw Values: Peak-Peak:{ch['vpp_raw']} | "
                        f"Avg:{ch['vavg_raw']} | RMS:{ch['vrms_raw']}\n"
                        f"      ⚠️ Note: Raw ADC values (0-255), not calibrated voltages\n"
                        f"{'-'*80}\n"
                    )
                else:
                    channel_display += f"CH{i}: ❌ OFF\n{'-'*80}\n"
            
            channel_text.set(channel_display)
            
        except Exception as e:
            log_debug(f"Main loop error: {e}")
            scpi_data['connected'] = False
            scpi_data['errors'] += 1
            scope = None  # Force reconnection
        
        time.sleep(INTERVALL_SCPI)

# Start threads
log_debug("Starting RIGOL MSO5000 Live Monitor")
threading.Thread(target=screenshot_loop, daemon=True).start()
threading.Thread(target=scpi_loop, daemon=True).start()

# Start GUI
update_image()
root.mainloop()